﻿using System;
using System.ComponentModel;
using System.Collections.ObjectModel;
using System.Collections.Specialized;

namespace LoadingDataDemo
{
    public delegate void NotifyPanlChanged();

    public class LstBoxModel : INotifyPropertyChanged, INotifyCollectionChanged
    {
        public LstBoxModel()
        {
            SelfMore = new MoreButonVis();
            SelfMore.PropertyChanged += new PropertyChangedEventHandler(SelfMore_PropertyChanged);
            SelfLstModel = new ObservableCollection<SimpleModel>();
        }

        void SelfMore_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            NotifyPropertyChanged("More");
        }

        #region attribute

        /// <summary>
        /// whether or not the data is loaded 
        /// </summary>
        public bool IsDataLoaded { get; private set; }

        /// <summary>
        /// ListBox's Itemsource
        /// </summary>
        private ObservableCollection<SimpleModel> SelfLstModel = null;
        public ObservableCollection<SimpleModel> LstModel
        {
            get 
            {
                if (SelfLstModel != null)
                    More.AppLstModelCount = SelfLstModel.Count;
                return SelfLstModel;
            }
            set 
            {
                SelfLstModel = value;
                NotifyPropertyChanged("LstModel");
            }
        }

        /// <summary>
        /// The more Button show 
        /// </summary>
        private MoreButonVis SelfMore = null;
        public MoreButonVis More
        {
            get { return SelfMore; }
            set 
            {
                SelfMore = value;
                NotifyPropertyChanged("More");
            }
        }
        
        #endregion //attribute

        #region overwritten INotifyPropertyChanged

        public event PropertyChangedEventHandler PropertyChanged;
        protected void NotifyPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        public event NotifyCollectionChangedEventHandler CollectionChanged;
        protected void NotifyCollectionChanged(string collectionName)
        {
            if (CollectionChanged != null)
            {
                CollectionChanged(this, new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Replace));
            }
        }

        #endregion //INotifyPropertyChanged

        /// <summary>
        /// Load data
        /// </summary>
        public void LoadData()
        {
            for (int i = 0; i < 100; i++)
            {
                LstModel.Add(new SimpleModel() { Name = "第" + i + "项", Discription = "这是第" + i + "项数据" });
            }

            IsDataLoaded = true;
        }
    }

    public class MoreButonVis : INotifyPropertyChanged
    {
        public NotifyPanlChanged NofityChangedPanl;

        #region attribute

        private int SelfAppLstModelCount = 0;
        public int AppLstModelCount
        {
            get { return SelfAppLstModelCount; }
            set
            {
                if (SelfAppLstModelCount != value)
                {
                    SelfAppLstModelCount = value;
                    if (NofityChangedPanl != null)
                        NofityChangedPanl();
                    NotifyPropertyChanged("AppLstModelCount");
                }
            }
        }

        private int SelfPageLstModelCount = 0;
        public int PageLstModelCount //是个索引值
        {
            get { return SelfPageLstModelCount; }
            set
            {
                if (SelfPageLstModelCount != value)
                {
                    SelfPageLstModelCount = value;
                    if (NofityChangedPanl != null)
                        NofityChangedPanl();
                    NotifyPropertyChanged("PageLstModelCount");
                }
            }
        }

        #endregion //attribute

        #region overwritten INotifyPropertyChanged

        public event PropertyChangedEventHandler PropertyChanged;
        protected void NotifyPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        #endregion //INotifyPropertyChanged
    }
}
